import re

from pathlib import Path
from nonebot import require
from nonebot.plugin.on import on_command
from nonebot.permission import SUPERUSER
from nonebot_plugin_uninfo import (
    get_session,
    Uninfo
)
from nonebot_plugin_alconna import (
    on_alconna,
    AlconnaMatcher,
    Match
)
from arclet.alconna import (
    Alconna,
    Args,
    Subcommand,
    Arparma,
    Option
)
from nonebot_plugin_alconna.uniseg import (
    UniMessage,
    Image,
    Text
)

from .stats import (
    get_level,
    get_stats_image
)
from .pve import screenshot_vb_img, vb_file
from .shop import screenshot_shop_img, shop_file

timewindow_prefix = ["生涯", ""]
name_args = Args["name?", str]


battle_pass = on_alconna(
    Alconna(timewindow_prefix, "季卡", name_args)
)

stats = on_alconna(
    Alconna(timewindow_prefix, '战绩', name_args)
)

@battle_pass.handle()
@stats.handle()
async def _(
    matcher: AlconnaMatcher,
    session: Uninfo,
    name: Match[str]
):
    if name.available:
        matcher.set_path_arg('name', name.result)
        return
    # 获取群昵称
    if not session.member or not session.member.nick:
        return
    pattern = r'(?:id:|id\s)(.+)'
    if match := re.match(
        pattern,
        session.member.nick,
        re.IGNORECASE
    ):
        matcher.set_path_arg('name', match.group(1))
        
        
name_prompt = UniMessage.template("{:At(user, $event.get_user_id())} 请发送游戏名称\n群昵称设置如下可快速查询:\n    id:name\n    ID name")

@battle_pass.got_path('name', prompt=name_prompt)
async def _(arp: Arparma, name: str):
    header = arp.header_match.result
    receipt = await UniMessage.text(f'正在查询 {name} 的{header}，请稍后...').send()
    level_info = await get_level(name, header)
    await battle_pass.send(level_info)
    await receipt.recall(delay=1)

@stats.got_path('name', prompt=name_prompt)
async def _(arp: Arparma, name: str):
    header = arp.header_match.result
    receipt = await UniMessage.text(f'正在查询 {name} 的{header}，请稍后...').send()
    res = await get_stats_image(name, header)
    if isinstance(res, Path):
        res = await UniMessage(Image(path=res)).export()
    await stats.send(res)
    await receipt.recall(delay=1)

shop = on_command('商城')

@shop.handle()
async def _():
    await shop.send(await UniMessage(Image(path=shop_file) + Text('https://www.fortnite.com/item-shop?lang=zh-Hans')).export())
    # await shop.finish('https://www.fortnite.com/item-shop?lang=zh-Hans' + "\n\n" + 'https://fortnite.gg/shop')
    
update_shop = on_command('更新商城', permission=SUPERUSER)

@update_shop.handle()
async def _():
    try:
        receipt = await UniMessage.text("正在更新商城，请稍后...").send()
        file = await screenshot_shop_img()
        await update_vb.send(await UniMessage(Text('手动更新商城成功') + Image(path=file)).export())
    except Exception as e:
        await update_vb.send(f'手动更新商城失败 | {e}')
    finally:
        await receipt.recall(delay=1)
        
vb = on_command('vb图', aliases={"VB图"})

@vb.handle()
async def _():
    await vb.finish(await UniMessage(Image(path=vb_file)).export())
    
update_vb = on_command('更新vb图', permission=SUPERUSER)

@update_vb.handle()
async def _():
    try:
        receipt = await UniMessage.text("正在更新vb图，请稍后...").send()
        file = await screenshot_vb_img()
        await update_vb.send(await UniMessage(Text('手动更新vb图成功') + Image(path=file)).export())
    except Exception as e:
        await update_vb.send(f'手动更新vb图失败 | {e}')
    finally:
        await receipt.recall(delay=1)


from functools import wraps

def exception_handler():
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            res = None
            try:
                res = await func(*args, **kwargs)
            except Exception as e:
                e = str(e)
                if "public" in e:
                    res = "战绩未公开"
                elif "exist" in e:
                    res = "用户不存在"
                elif "match" in e:
                    res = "该玩家当前赛季没有进行过任何对局"
                elif "timed out" in e:
                    res = "请求超时, 请稍后再试"
                elif "failed to fetch" in e:
                    res = "拉取账户信息失败, 稍后再试"
                else:
                    res = f"未知错误: {e}"
            finally:
                return res
        return wrapper
    return decorator



from pydantic import BaseModel
from typing import Optional
from nonebot import get_plugin_config

from pathlib import Path
import nonebot_plugin_localstore as store


class Config(BaseModel):
    fortnite_api_key: Optional[str] = ""
    
fconfig: Config = get_plugin_config(Config)

cache_dir: Path = store.get_plugin_cache_dir()
data_dir: Path = store.get_plugin_data_dir()


import asyncio
from PIL import Image
from pathlib import Path
from playwright.async_api import async_playwright
from .config import data_dir, cache_dir

vb_file = data_dir / "vb.png"
hot_info_1_path = cache_dir / 'hot_info_1.png'
container_hidden_xs_path = cache_dir / 'container_hidden_xs.png'
hot_info_2_path = cache_dir / 'hot_info_2.png'

async def screenshot_vb_img() -> Path:
    url = "https://freethevbucks.com/timed-missions"
    
    try:
        browser = None
        async with async_playwright() as p:
            browser = await p.chromium.launch(headless=True)
            page = await browser.new_page()
            await page.goto(url)
            
            # 截图函数，超时则跳过
            async def take_screenshot(locator, path):
                try:
                    # 检查元素内容是否为空
                    content = await locator.inner_html()
                    if content.strip():  # 如果内容不为空
                        await asyncio.wait_for(locator.screenshot(path=path), timeout=5)
                    else:
                        logger.warning(f"Locator for {path.name} is empty.")
                except Exception:
                    pass
                    
            
            # 截取第一个 <div class="hot-info">
            hot_info_1 = page.locator('div.hot-info').nth(0)
            await take_screenshot(hot_info_1, hot_info_1_path)
            
            # 截取 <div class="container hidden-xs">
            container_hidden_xs = page.locator('div.container.hidden-xs')
            await take_screenshot(container_hidden_xs, container_hidden_xs_path)
            
            # 截取第二个 <div class="hot-info">
            hot_info_2 = page.locator('div.hot-info').nth(1)
            await take_screenshot(hot_info_2, hot_info_2_path)
            
            combine_imgs()
    finally:
        if browser:
            await browser.close()
    
    return vb_file

def combine_imgs():
    # 打开截图文件（如果存在）
    combined_image = None
    img_paths = [hot_info_1_path, container_hidden_xs_path, hot_info_2_path]
    img_paths = [i for i in img_paths if i.exists()]
    if not img_paths:
        raise Exception('所有选择器的截图文件均不存在')
    try:
        images = [Image.open(img_path) for img_path in img_paths]
        # 获取尺寸并创建新图像
        widths, heights = zip(*(img.size for img in images))
        total_width = max(widths)
        total_height = sum(heights)
        combined_image = Image.new('RGB', (total_width, total_height))
        
        # 将截图粘贴到新图像中
        y_offset = 0
        for img in images:
            combined_image.paste(img, (0, y_offset))
            y_offset += img.height
        
        # 保存合并后的图像
        combined_image.save(vb_file)
    finally:
        # 关闭并删除所有截图文件
        for img in images:
            img.close()
        for img_path in img_paths:
            img_path.unlink()
        if combined_image:
            combined_image.close()



import httpx
import asyncio

from PIL import Image
from pathlib import Path
from nonebot.log import logger
from playwright.async_api import async_playwright

from .config import data_dir

shop_file = data_dir / "shop.png"

async def screenshot_shop_img() -> Path:
    # url = "https://www.fortnite.com/item-shop?lang=zh-Hans"
    url = "https://fortnite.gg/shop"
    
    headers = {
      'User-Agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36",
      'Accept': "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
      'Accept-Encoding': "gzip, deflate",
      'upgrade-insecure-requests': "1",
      'dnt': "1",
      'x-requested-with': "mark.via",
      'sec-fetch-site': "none",
      'sec-fetch-mode': "navigate",
      'sec-fetch-user': "?1",
      'sec-fetch-dest': "document",
      'accept-language': "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
      'Cookie': "_sharedid=f02028dd-dce2-4b07-bba9-301d54e68dbd; _sharedid_cst=zix7LPQsHA%3D%3D; _lr_env_src_ats=false; hb_insticator_uid=799b5897-b5a3-48c4-a46f-8bb8bf9082ac"
    }
        
    # token = await cf_token()
    # logger.info(token)
    browser = None
    try:
        async with async_playwright() as p:
            browser = await p.chromium.launch(headless=True)
            context = await browser.new_context(extra_http_headers = headers)
            page = await context.new_page()
            # page.on('requestfailed', lambda request: logger.warning(f'Request failed: {request.url}'))
            await page.add_style_tag(content='* { transition: none !important; animation: none !important; }')
            await page.goto(url)

            async def wait_for_load():
                await page.wait_for_load_state('networkidle', timeout=90000)            
            
            async def scroll_page():
                for _ in range(20):
                    await page.evaluate("""() => {
                        window.scrollBy(0, document.body.scrollHeight / 20);
                    }""")
                    await asyncio.sleep(1)  # 等待1秒以加载内容
            
            await asyncio.gather(
                wait_for_load(),
                scroll_page()
            )
            
            await page.screenshot(path=shop_file, full_page=True)
            return shop_file
    finally:
        await browser.close()




from nonebot import (
    require,
    get_driver, # @get_driver().on_startup 装饰启动时运行函数
    get_bots   # dict[str, BaseBot]
)
from nonebot.log import logger
from nonebot.plugin import PluginMetadata
from nonebot.plugin.load import inherit_supported_adapters

require("nonebot_plugin_uninfo")
require("nonebot_plugin_alconna")
require("nonebot_plugin_apscheduler")
require("nonebot_plugin_localstore")
from nonebot_plugin_apscheduler import scheduler

from .config import Config
from .matcher import *
from .pve import screenshot_vb_img
from .shop import screenshot_shop_img

__plugin_meta__ = PluginMetadata(
    name="堡垒之夜游戏插件",
    description="堡垒之夜战绩，季卡，商城，vb图查询",
    usage="季卡/生涯季卡/战绩/生涯战绩/商城/vb图",
    type="application",
    config=Config,
    homepage="https://github.com/fllesser/nonebot-plugin-fortnite",
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna", "nonebot_plugin_uninfo")
)


@scheduler.scheduled_job(
    "cron",
    id = 'fortnite',
    hour = 8,
    minute = 5,
)
async def _():
    try:
        await screenshot_shop_img()
    except Exception as e:
        logger.warning(f'商城更新失败: {e}')
    try:
        await screenshot_vb_img()
    except Exception:
        logger.warning(f'vb图更新失败: {e}')


import httpx
import asyncio

from io import BytesIO
from pathlib import Path
from nonebot import get_driver
from nonebot.log import logger

from PIL import (
    Image,
    ImageFont, 
    ImageDraw
)

from .config import (
    fconfig,
    cache_dir, 
    data_dir
)

from fortnite_api import (
    Client,
    BrPlayerStats,
    StatsImageType,
    TimeWindow
)

from .other import exception_handler

api_key = fconfig.fortnite_api_key

async def get_stats(
    name: str, 
    time_window: TimeWindow = TimeWindow.SEASON,
    image_type: StatsImageType = None
) -> BrPlayerStats:
    params = {
        'name': name,
        'time_window': time_window
    }
    if image_type:
        params['image'] = image_type
    async with Client(api_key=api_key) as client:
        return await client.fetch_br_stats(**params)

@exception_handler()        
async def get_level(name: str, time_window: str) -> int:
    time_window = TimeWindow.LIFETIME if time_window.startswith("生涯") else TimeWindow.SEASON
    stats = await get_stats(name, time_window)
    bp = stats.battle_pass
    return f'{stats.user.name}: Lv{bp.level} | {bp.progress}% to Lv{bp.level + 1}'

@exception_handler()
async def get_stats_image(name: str, time_window: str) -> Path:
    time_window = TimeWindow.LIFETIME if time_window.startswith("生涯") else TimeWindow.SEASON
    stats = await get_stats(name, time_window, StatsImageType.ALL)
    return await get_stats_img_by_url(stats.image.url, stats.user.name)
    

font_path: Path | None = None

@get_driver().on_startup
async def _():
    hans = data_dir / "SourceHanSansSC-Bold-2.otf"
    global font_path
    if hans.exists():
        font_path = hans
        logger.info(f'战绩绘图将使用字体: {font_path.name}')
    else:
        logger.warning(f"请前往仓库下载字体到 {data_dir}/，否则战绩查询可能无法显示中文名称")
    
async def get_stats_img_by_url(url: str, name: str) -> Path:
    file = cache_dir / f"{name}.png"
    async with httpx.AsyncClient() as client:
        resp = await client.get(url)
        resp.raise_for_status()
        
    with open(file, "wb") as f:
        f.write(resp.content)
    # 如果不包含中文名，返回
    if not font_path or not contains_chinese(name):
        return file
    
    with Image.open(file) as img:
        draw = ImageDraw.Draw(img)

        # 矩形区域的坐标
        left, top, right, bottom = 26, 90, 423, 230

        # 创建渐变色并填充矩形区域
        width = right - left
        height = bottom - top
        
        start_color = (0, 33, 69, 255)
        end_color = (0, 82, 106, 255)
        for i in range(width):
            for j in range(height):
                r = int(start_color[0] + (end_color[0] - start_color[0]) * (i + j) / (width + height))
                g = int(start_color[1] + (end_color[1] - start_color[1]) * (i + j) / (width + height))
                b = int(start_color[2] + (end_color[2] - start_color[2]) * (i + j) / (width + height))
                draw.point((left + i, top + j), fill=(r, g, b))
        
        # 指定字体
        font_size = 36
        # hansans = data_dir / "SourceHanSansSC-Bold-2.otf"
        font = ImageFont.truetype(font_path, font_size)
        
        # 计算字体坐标
        length = draw.textlength(name, font=font)
        x = left + (right - left - length) / 2
        y = top + (bottom - top - font_size) / 2
        draw.text((x, y), name, fill = "#fafafa", font = font)
        
        # 保存
        img.save(file)
        return file
    
def contains_chinese(text):
    import re
    pattern = re.compile(r'[\u4e00-\u9fff]')
    return bool(pattern.search(text))
